/****************************************************************************
   VDDDEEXT.c - 
   Copyright 1995-1997 Ventana Systems, Inc.
   Example external function file definitions using the
   DDE API for Vensim
   with Vensim 

   See also Venext.dll for more information */
#include "C:\Users\Public\Vensim\6.0b\DSS\comp\vensim.h"
GLOB_VARS *VENGV ; /* the value for this is set by set_gv below */
#include <stdlib.h>



/*******************************************
 1 - function ids - used to swich between choices 
 *******************************************/
#define ADD_FUNC 0
#define ADD_VECTOR_FUNC 1

/*******************************************
 2 - function prototypes 
*********************************************/
double ADD(double x,double y) ;
double ADD_VECTOR(float *added_vec,float *avec,float *bvec,int nelm) ;

/****************************************************
 3 - Grouping of functions in a structure
 ****************************************************
 ***********************************************************/

static FUNC_DESC Flist[] = {
                    {"ADD"," {A} , {B} ",2,0,ADD_FUNC,0,0,0,0},
                    {"ADD_VECTOR"," {A} , {B} , {Nelm} ",3,2,ADD_VECTOR_FUNC,1,0,0,0},
                    {'\0',0,0,0}} ;


 /**************************
  This function is a version check - it is required for Vensim 3.0 
  and beyond - external function libraries are not compatible between
  Vensim version 1.6 and 3.0 */

int VEFCC version_info()
 {
  return(EXTERNAL_VERSION) ;
  }
/* if you make changes to functions you should update this so that Vensim will recognize
   that there is a mismatch and retranslation is necessary */
unsigned short VEFCC funcversion_info()
{
   return(1) ;
   }

int VEFCC set_gv(GLOB_VARS *vgv) 
 {
  VENGV = vgv ;
  if(!VENGV || 
     VENGV->vgv_magic_start != VGV_MAGIC_START ||
     VENGV->vgv_magic_end != VGV_MAGIC_END)
     return 0 ;
  return 1 ;
  }


/****************************************************************
 5 External function definitions
 *****************************************************************/

int VEFCC user_definition(int i,char **sym,char **descript,
                       int *num_arg,int *num_vector,
                       int *func_index,int *dim_act,
                       int *modify,int *num_loop,
                       int *num_literal,int *num_lookup)
 {
  if(Flist[i].sym) {
     *sym = Flist[i].sym ;
     *descript = Flist[i].argument_desc ;
     *num_arg = Flist[i].num_args ;
     *num_vector = Flist[i].num_vector ;
     *func_index = Flist[i].func_index ;
     *modify = Flist[i].modify ;
     *num_loop = Flist[i].num_loop ;
     *num_literal = Flist[i].num_literal ;
     *num_lookup = Flist[i].num_lookup ;
     return(1) ;
     }
  return(0) ; /* indicating the end of the list */
  }

/***********************************************
 5a Initialization and termination code - called before and after each simulation
 ************************************************/
static int Dde_channel ;
int VEFCC simulation_setup(int iniflag)
 { 
  if(!iniflag)
     return(1) ; /* only do initialization on iniflag */
  Dde_channel = (*VENGV->vdde_initiate)("Excel","VDDEEXT.XLS") ;
  if(Dde_channel < 0) {
     (*VENGV->error_message)(VERROR,"Unable to open DDE channel to Excel - VDDEEXT") ;
     }
  return(1) ;
  }
int VEFCC simulation_shutdown(int finalflag)
 {  
  if(!finalflag)
     return(1) ;
  if(Dde_channel >= 0)
     (*VENGV->vdde_terminate)(Dde_channel) ; 
  Dde_channel = -1 ;
  return(1) ;
  }    

/*********************************************************************
 6 - vensim_external - the actual external function call
 ******************************************************************/

int VEFCC vensim_external(VV *val,int nval,int funcid)
 {
  double rval ;
  int n ;
  switch(funcid) {
     case ADD_FUNC : /* simple function - call with double return double */
       rval = ADD(val[0].val,val[1].val) ;
       break ;
     case ADD_VECTOR_FUNC :
       n = (int)(val[3].val+.5) ;
       rval = ADD_VECTOR(val[0].vec->vals,val[1].vec->vals,val[2].vec->vals,n) ;
       break ;
     default :
       return(0) ; /* indicate an error condition to Vensim */
     }
  /* set val[0], this value will be used in the equation output */
  val[0].val = (float)rval ;
  return(1) ; /* a 1 return value signals vensim of successful completetion */
  }


/*************************************************************************
 7 - actual function bodies - these could be a separate file 
 *************************************************************************/
double ADD(double x,double y)
 {
  char str[20] ;
  char *buf ;
  buf = _gcvt(x,10,str) ;
  if(!(*VENGV->vdde_poke)(Dde_channel,"R1C1",buf)) {
     (*VENGV->error_message)(VERROR,"DDE failure sending data during ADD") ;
     return(-1) ;
     }
  buf = _gcvt(y,10,str) ;
  if(!(*VENGV->vdde_poke)(Dde_channel,"R1C2",buf)) {
     (*VENGV->error_message)(VERROR,"DDE failure sending data during ADD") ;
     return(-1) ;
     }
  if(!(*VENGV->vdde_wait_execute)(Dde_channel,"[CALCULATE.DOCUMENT()]",100)) {
     (*VENGV->error_message)(VERROR,"Calculate command not sent") ;
     return(-1) ;
     }
  if(!(*VENGV->vdde_request)(Dde_channel,"R1C3",&buf,500)) {
     (*VENGV->error_message)(VERROR,"DDE failure computing ADD") ;
     return(-1) ;
     }
  x = atof(buf) ;
  return(x) ;
  }
double ADD_VECTOR(float *added_vec,float *avec,float *bvec,int nelm)
 {
  int i ;
  char *outbuf ;
  char str[20] ;
  char *cellvec[8] ;
  char cellstr[8*8] ; /* buffer to word boundaries */
  char *buf[9] ;
  char *tv ;

  if(nelm > 8) {
     (*VENGV->error_message)(VERROR,"Need to increase buffer size in VDDEXT.C") ;
     return(-1) ;
     }
  for(tv = cellstr,i=0;i<nelm;i++,tv += 8) {
     cellvec[i] = tv ;
     tv[0] = 'R' ;
     tv[1] = '2' + i ;
     tv[2] = 'C' ;
     tv[3] = '1' ;
     tv[4] = '\0' ;
     outbuf = _gcvt(avec[i],10,str) ;
     if(!(*VENGV->vdde_poke)(Dde_channel,cellvec[i],outbuf)) {
        (*VENGV->error_message)(VERROR,"DDE failure sending data during ADD_VECTOR") ;
        return(-1) ;
        }
     tv[3] = '2' ;
     outbuf = _gcvt(bvec[i],10,str) ;
     if(!(*VENGV->vdde_poke)(Dde_channel,cellvec[i],outbuf)) {
        (*VENGV->error_message)(VERROR,"DDE failure sending data during ADD_VECTOR") ;
        return(-1) ;
        }
     tv[3] = '3' ; /* prepare for read */
     }
  if(!(*VENGV->vdde_wait_execute)(Dde_channel,"[CALCULATE.DOCUMENT()]",100)) {
     (*VENGV->error_message)(VERROR,"Calculate command not sent") ;
     return(-1) ;
     }
  /* now query the whole vector at once */
  if(!(*VENGV->vdde_vrequest)(Dde_channel,cellvec,buf,nelm,750)) {
     (*VENGV->error_message)(VERROR,"DDE failure computing ADD_VECTOR") ;
     return(-1) ;
     }
  for(i=0;i<nelm;i++) 
     added_vec[i] = (float)atof(buf[i]) ;
  return((double)added_vec[0]) ;
  }
